@@ -749,6 +749,7 @@ typedef struct {
 typedef struct {
 	cairo_matrix_t		matrix;
 	GpRegion*		clip;
+	cairo_matrix_t		clip_matrix;
 	GpCompositingMode    	composite_mode;
 	GpCompositingQuality 	composite_quality;
 	InterpolationMode 	interpolation;
@@ -769,6 +770,7 @@ typedef struct {
 	void            *image;
 	int             type; 
 	GpRegion*	clip;
+	GpMatrix*	clip_matrix;
 	GpRect		bounds;
 	GpUnit		page_unit;
 	float		scale;
@@ -1129,7 +1131,7 @@ GpStatus GdipSetSmoothingMode(GpGraphics *graphics, SmoothingMode mode);
 GpStatus GdipGetSmoothingMode(GpGraphics *graphics, SmoothingMode *mode);
 GpStatus GdipSetVisibleClip_linux (GpGraphics *graphics, GpRect *rect);
 GpStatus GdipTranslateClip (GpGraphics *graphics, float dx, float dy);
-GpStatus GdipTranslateClipI (GpGraphics *graphics, UINT dx, UINT dy);
+GpStatus GdipTranslateClipI (GpGraphics *graphics, int dx, int dy);
 
 /* Status */
 GpStatus gdip_get_status (cairo_status_t status);
@@ -1231,9 +1233,9 @@ GpStatus GdipTransformMatrixPointsI (GpMatrix *matrix, GpPoint *pts, int count);
 GpStatus GdipVectorTransformMatrixPoints (GpMatrix *matrix, GpPointF *pts, int count);
 GpStatus GdipVectorTransformMatrixPointsI (GpMatrix *matrix, GpPoint *pts, int count);
 GpStatus GdipGetMatrixElements (GpMatrix *matrix, float *matrixOut);
-GpStatus GdipIsMatrixInvertible (GpMatrix *matrix, int *result);
-GpStatus GdipIsMatrixIdentity (GpMatrix *matrix, int *result);
-GpStatus GdipIsMatrixEqual (GpMatrix *matrix, GpMatrix *matrix2, int *result);
+GpStatus GdipIsMatrixInvertible (GpMatrix *matrix, BOOL *result);
+GpStatus GdipIsMatrixIdentity (GpMatrix *matrix, BOOL *result);
+GpStatus GdipIsMatrixEqual (GpMatrix *matrix, GpMatrix *matrix2, BOOL *result);
 
 
 /* Font Family*/         
