@@ -24,7 +24,7 @@
  **/
 
 #include <math.h>
-#include "gdip.h"
+#include "matrix.h"
 
 /*
 	GDI+ matrix takes 6 elements arranged in 3 rows by 2 columns. The identity matrix is
@@ -63,9 +63,9 @@ gdip_is_matrix_empty (GpMatrix* matrix)
 		return TRUE;
 
 	/* compare the matrix elements with the empty (no-op) version */
-	return ((matrix->xx == 1.0f) && (matrix->yx == 0.0f) &&
-		(matrix->xy == 0.0f) && (matrix->yy == 1.0f) &&
-		(matrix->x0 == 0.0f) && (matrix->y0 == 0.0f));
+	return (gdip_near_one (matrix->xx) && gdip_near_zero (matrix->yx) &&
+		gdip_near_zero (matrix->xy) && gdip_near_one (matrix->yy) &&
+		gdip_near_zero (matrix->x0) && gdip_near_zero (matrix->y0));
 }
 
 
@@ -85,76 +85,116 @@ gdip_is_matrix_a_translation (GpMatrix *matrix)
 GpStatus 
 GdipCreateMatrix (GpMatrix **matrix)
 {
+	GpMatrix *result;
+
 	g_return_val_if_fail  (matrix != NULL, InvalidParameter);
 	
-	*matrix = GdipAlloc (sizeof(GpMatrix));
-	cairo_matrix_init (*matrix, 1, 0, 0, 1, 0, 0);
-	
+	result = GdipAlloc (sizeof (GpMatrix));
+	if (!result)
+		return OutOfMemory;
+
+	cairo_matrix_init_identity (result);
+
+	*matrix = result;
 	return Ok;
 }
 
 GpStatus
 GdipCreateMatrix2 (float m11, float m12, float m21, float m22, float dx, float dy, GpMatrix **matrix)
 {
+	GpMatrix *result;
+
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
-	*matrix = GdipAlloc (sizeof(GpMatrix));
-	cairo_matrix_init (*matrix, m11, m12, m21, m22, dx, dy);
+
+	result = GdipAlloc (sizeof (GpMatrix));
+	if (!result)
+		return OutOfMemory;
+
+	cairo_matrix_init (result, m11, m12, m21, m22, dx, dy);
+
+	*matrix = result;
 	return Ok;
 }
 
 GpStatus
 GdipCreateMatrix3 (const GpRectF *rect, const GpPointF *dstplg, GpMatrix **matrix)
 {
-	double m11, m12, m21, m22, dx, dy;
+	GpMatrix *result;
+	GpPointF *p, *p0, *p1, *p2;
+	double m11, m12, m21, m22;
 
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 	g_return_val_if_fail (dstplg != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 
-	m11 = rect->X;
-	m12 = rect->Y;
-	m21 = rect->Width;
-	m22 = rect->Height;
-	dx = dstplg->X;
-	dy = dstplg->Y;
+	p = (GpPointF*) dstplg;
+	p0 = p++;
+	p1 = p++;
+	p2 = p;
 
-	*matrix = GdipAlloc (sizeof (GpMatrix));
-	cairo_matrix_init (*matrix, m11, m12, m21, m22, dx, dy);
+	m11 = (p1->X - p0->X) / rect->Width;
+	m12 = (p1->Y - p0->Y) / rect->Width;
+	m21 = (p2->X - p0->X) / rect->Height;
+	m22 = (p2->Y - p0->Y) / rect->Height;
 
+	result = GdipAlloc (sizeof (GpMatrix));
+	if (!result)
+		return OutOfMemory;
+
+	cairo_matrix_init (result, m11, m12, m21, m22, p0->X, p0->Y);
+	cairo_matrix_translate (result, -rect->X, -rect->Y);
+
+	*matrix = result;
 	return Ok;
 }
 
 GpStatus
 GdipCreateMatrix3I (const GpRect *rect, const GpPoint *dstplg, GpMatrix **matrix)
 {
-	double m11, m12, m21, m22, dx, dy;
+	GpMatrix *result;
+	GpPoint *p, *p0, *p1, *p2;
+	double m11, m12, m21, m22;
 
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 	g_return_val_if_fail (dstplg != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 
-	m11 = rect->X;
-	m12 = rect->Y;
-	m21 = rect->Width;
-	m22 = rect->Height;
-	dx = dstplg->X;
-	dy = dstplg->Y;
+	p = (GpPoint*) dstplg;
+	p0 = p++;
+	p1 = p++;
+	p2 = p;
 
-	*matrix = GdipAlloc (sizeof (GpMatrix));
-	cairo_matrix_init (*matrix, m11, m12, m21, m22, dx, dy);
+	m11 = (p1->X - p0->X) / (double)rect->Width;
+	m12 = (p1->Y - p0->Y) / (double)rect->Width;
+	m21 = (p2->X - p0->X) / (double)rect->Height;
+	m22 = (p2->Y - p0->Y) / (double)rect->Height;
 
+	result = GdipAlloc (sizeof (GpMatrix));
+	if (!result)
+		return OutOfMemory;
+
+	cairo_matrix_init (result, m11, m12, m21, m22, p0->X, p0->Y);
+	cairo_matrix_translate (result, -rect->X, -rect->Y);
+
+	*matrix = result;
         return Ok;
 }
 
 GpStatus
 GdipCloneMatrix (GpMatrix *matrix, GpMatrix **cloneMatrix)
 {
+	GpMatrix *result;
+
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 	g_return_val_if_fail (cloneMatrix != NULL, InvalidParameter);
 
-	*cloneMatrix = GdipAlloc (sizeof (GpMatrix));
-        gdip_cairo_matrix_copy (*cloneMatrix, matrix);
+	result = GdipAlloc (sizeof (GpMatrix));
+	if (!result)
+		return OutOfMemory;
+
+        gdip_cairo_matrix_copy (result, matrix);
 	
+	*cloneMatrix = result;
 	return Ok;
 }
 
@@ -356,14 +396,16 @@ GdipVectorTransformMatrixPointsI (GpMatrix *matrix, GpPoint *pts, int count)
 }
 
 GpStatus 
-GdipIsMatrixInvertible (GpMatrix *matrix, int *result)
+GdipIsMatrixInvertible (GpMatrix *matrix, BOOL *result)
 {
-        cairo_status_t status;
+	cairo_status_t status;
+	cairo_matrix_t copy;
 
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 	g_return_val_if_fail (result != NULL, InvalidParameter);
 
-	status = cairo_matrix_invert (matrix);
+	gdip_cairo_matrix_copy (&copy, matrix);
+	status = cairo_matrix_invert (&copy);
 
         if (status == CAIRO_STATUS_INVALID_MATRIX)
                 *result = 0;
@@ -385,20 +427,18 @@ matrix_equals (GpMatrix *x, GpMatrix *y)
 }
 
 GpStatus
-GdipIsMatrixIdentity (GpMatrix *matrix, int *result)
+GdipIsMatrixIdentity (GpMatrix *matrix, BOOL *result)
 {
-	Status s;
-        GpMatrix identity;
+	if (!matrix || !result)
+		return InvalidParameter;
 
-        cairo_matrix_init_identity (&identity);
-
-        s = GdipIsMatrixEqual (matrix, &identity, result);
-
-        return s;
+	/* note: we CAN'T use GdipIsMatrixEqual - as the precision is different */
+        *result = gdip_is_matrix_empty (matrix);
+        return Ok;
 }
 
 GpStatus
-GdipIsMatrixEqual (GpMatrix *matrix, GpMatrix *matrix2, int *result)
+GdipIsMatrixEqual (GpMatrix *matrix, GpMatrix *matrix2, BOOL *result)
 {
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix2 != NULL, InvalidParameter);
